@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2015 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2008-2017 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -464,6 +464,7 @@ int vpip_time_units_from_handle(vpiHandle obj)
       struct __vpiSysTaskCall*task;
       struct __vpiScope*scope;
       struct __vpiSignal*signal;
+      __vpiNamedEvent*event;
 
       if (obj == 0)
 	    return vpip_get_time_precision();
@@ -483,6 +484,11 @@ int vpip_time_units_from_handle(vpiHandle obj)
 	    scope = vpip_scope(signal);
 	    return scope->time_units;
 
+	  case vpiNamedEvent:
+	    event = dynamic_cast<__vpiNamedEvent*>(obj);
+	    scope = event->get_scope();
+	    return scope->time_units;
+
 	  default:
 	    fprintf(stderr, "ERROR: vpip_time_units_from_handle called with "
 		    "object handle type=%u\n", obj->get_type_code());
@@ -1067,13 +1073,13 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
 	    vvp_time64_t dly;
 	    int scale;
 
-            if (vpi_get(vpiAutomatic, obj)) {
-                  fprintf(stderr, "vpi error: cannot put a value with "
-                                  "a delay on automatically allocated "
-                                  "variable '%s'\n",
-                                  vpi_get_str(vpiName, obj));
-                  return 0;
-            }
+	    if (vpi_get(vpiAutomatic, obj)) {
+		  fprintf(stderr, "VPI error: cannot put a value with "
+				  "a delay on automatically allocated "
+				  "variable '%s'.\n",
+				  vpi_get_str(vpiName, obj));
+		  return 0;
+	    }
 
 	    assert(when != 0);
 
@@ -1095,9 +1101,22 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
 		  break;
 	    }
 
+	    if ((dly == 0) && schedule_at_rosync()) {
+		  fprintf(stderr, "VPI error: attempted to put a value to "
+				  "variable '%s' during a read-only synch "
+				  "callback.\n", vpi_get_str(vpiName, obj));
+		  return 0;
+	    }
+
 	    vpip_put_value_event*put = new vpip_put_value_event;
 	    put->handle = obj;
-	    put->value = *vp;
+	    if (dynamic_cast<__vpiNamedEvent*>(obj)) {
+		  put->value.format = vpiIntVal;
+		  put->value.value.integer = 0;
+	    } else {
+		  assert(vp);
+		  put->value = *vp;
+	    }
 	      /* Since this is a scheduled put event we must copy any pointer
 	       * data to keep it available until the event is actually run. */
 	    switch (put->value.format) {
@@ -1132,6 +1151,13 @@ vpiHandle vpi_put_value(vpiHandle obj, s_vpi_value*vp,
 	    return 0;
       }
 
+      if (schedule_at_rosync()) {
+            fprintf(stderr, "VPI error: attempted to put a value to "
+			    "variable '%s' during a read-only synch "
+			    "callback.\n", vpi_get_str(vpiName, obj));
+            return 0;
+      }
+
       obj->vpi_put_value(vp, flags);
 
       return 0;
